@@ -14,17 +14,50 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 import json
-from robotframework_debug_adapter.constants import DEBUG
+
+
 from robocorp_ls_core.debug_adapter_core.dap import dap_base_schema as base_schema
+from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import BaseSchema
+from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+    Breakpoint,
+    ConfigurationDoneRequest,
+    ConfigurationDoneResponse,
+    ContinueRequest,
+    DisconnectRequest,
+    EvaluateRequest,
+    InitializeRequest,
+    InitializedEvent,
+    LaunchRequest,
+    LaunchResponse,
+    NextRequest,
+    PauseRequest,
+    ScopesRequest,
+    SetBreakpointsRequest,
+    SetBreakpointsResponseBody,
+    SetExceptionBreakpointsRequest,
+    SourceBreakpoint,
+    StackTraceRequest,
+    StepInRequest,
+    StepOutRequest,
+    ThreadsRequest,
+    VariablesRequest,
+)
 from robocorp_ls_core.robotframework_log import get_logger
+from robotframework_debug_adapter.constants import DEBUG
+from typing import Optional
+
 
 log = get_logger(__name__)
 
 
 class DebugAdapterComm(object):
     """
-    This is the class that actually processes commands.
+    This is the class that actually processes commands from the client (VSCode)
+    using the process stdin/stdout.
+    
+    It's responsible for actually starting the debugger process later on.
 
     It's created in the main thread and then control is passed on to the reader thread so that whenever
     something is read the json is handled by this processor.
@@ -35,10 +68,19 @@ class DebugAdapterComm(object):
     """
 
     def __init__(self, write_to_client_queue):
+        from robotframework_debug_adapter.launch_process import LaunchProcess
+
         self.write_to_client_queue = write_to_client_queue
-        self._launch_process = None  # : :type self._launch_process: LaunchProcess
+        self._launch_process: Optional[LaunchProcess] = None
         self._supports_run_in_terminal = False
         self._initialize_request_arguments = None
+        self._run_in_debug_mode = True
+
+        # i.e.: When a backend receives a stopped event, it sets itself as
+        # a weakreference in weak_stopped_target_comm (so that we can redirect
+        # related events, such as evaluate, stack trace, etc to the proper
+        # backend).
+        self.weak_stopped_target_comm = lambda: None
 
     @property
     def supports_run_in_terminal(self):
@@ -49,7 +91,6 @@ class DebugAdapterComm(object):
         return self._initialize_request_arguments
 
     def from_client(self, protocol_message):
-
         from robocorp_ls_core.debug_adapter_core.debug_adapter_threads import (
             READER_THREAD_STOPPED,
         )
@@ -77,31 +118,37 @@ class DebugAdapterComm(object):
                         % (method_name, self.__class__.__name__)
                     )
 
-    def on_initialize_request(self, request):
-        """
-        :param InitializeRequest request:
-        """
-        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import build_response
+    def on_initialize_request(self, request: InitializeRequest):
+        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import (
+            build_response,
+        )
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+            InitializeResponse,
+        )
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import Capabilities
 
-        # : :type initialize_response: InitializeResponse
-        # : :type body: Capabilities
         self._initialize_request_arguments = request.arguments
-        initialize_response = build_response(request)
+        initialize_response: InitializeResponse = build_response(request)
         self._supports_run_in_terminal = request.arguments.supportsRunInTerminalRequest
-        body = initialize_response.body
+        body: Capabilities = initialize_response.body
         body.supportsConfigurationDoneRequest = True
         self.write_to_client_message(initialize_response)
 
-    def on_launch_request(self, request):
-        """
-        :param LaunchRequest request:
-        """
-        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import build_response
+    def on_launch_request(self, request: LaunchRequest):
+        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import (
+            build_response,
+        )
         from robotframework_debug_adapter.launch_process import LaunchProcess
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import InitializedEvent
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+            SetDebuggerPropertyRequest,
+            SetDebuggerPropertyArguments,
+            AttachRequest,
+            AttachRequestArguments,
+        )
 
-        # : :type launch_response: LaunchResponse
-        launch_response = build_response(request)
+        self._run_in_debug_mode = run_in_debug_mode = not request.arguments.noDebug
+
+        launch_response: LaunchResponse = build_response(request)
         launch_process = None
         try:
 
@@ -123,17 +170,37 @@ class DebugAdapterComm(object):
             launch_response.success = False
             launch_response.message = str(e)
 
-        self.write_to_client_message(launch_response)  # acknowledge it
-        if launch_process is not None:
-            launch_process.after_launch_response_sent()
+        def on_finish_setup(*args, **kwargs):
+            self.write_to_client_message(launch_response)  # acknowledge it
+            if launch_process is not None:
+                launch_process.after_launch_response_sent()
 
-    def on_configurationDone_request(self, request):
-        """
-        :param ConfigurationDoneRequest request:
-        """
-        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import build_response
+        if run_in_debug_mode and launch_process:
+            # We must configure/attach to pydevd
+            launch_process.write_to_pydevd(
+                SetDebuggerPropertyRequest(
+                    SetDebuggerPropertyArguments(
+                        skipSuspendOnBreakpointException=("BaseException",),
+                        skipPrintBreakpointException=("NameError",),
+                        multiThreadsSingleNotification=True,
+                    )
+                )
+            )
 
-        configuration_done_response = build_response(request)
+            attach_request_arguments = AttachRequestArguments(justMyCode=False)
+            pydevd_attach_request = AttachRequest(attach_request_arguments)
+            launch_process.write_to_pydevd(
+                pydevd_attach_request, on_response=on_finish_setup
+            )
+        else:
+            on_finish_setup()
+
+    def on_configurationDone_request(self, request: ConfigurationDoneRequest):
+        from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import (
+            build_response,
+        )
+
+        configuration_done_response: ConfigurationDoneResponse = build_response(request)
         launch_process = self._launch_process
         if launch_process is None:
             configuration_done_response.success = False
@@ -144,7 +211,6 @@ class DebugAdapterComm(object):
             return
 
         if launch_process.send_and_wait_for_configuration_done_request():
-            # : :type configuration_done_response: ConfigurationDoneResponse
             self.write_to_client_message(configuration_done_response)  # acknowledge it
 
         else:
@@ -155,11 +221,7 @@ class DebugAdapterComm(object):
             )
             self.write_to_client_message(configuration_done_response)
 
-    def on_disconnect_request(self, request):
-        """
-        :param DisconnectRequest request:
-        """
-        # : :type disconnect_response: DisconnectResponse
+    def on_disconnect_request(self, request: DisconnectRequest):
         disconnect_response = base_schema.build_response(request)
 
         if self._launch_process is not None:
@@ -167,40 +229,39 @@ class DebugAdapterComm(object):
 
         self.write_to_client_message(disconnect_response)
 
-    def on_pause_request(self, request):
-        """
-        :param PauseRequest request:
-        """
-        # : :type pause_response: PauseResponse
-        pause_response = base_schema.build_response(request)
-        self.write_to_client_message(pause_response)
+    def on_pause_request(self, request: PauseRequest):
+        if self._run_in_debug_mode and self._launch_process is not None:
+            self._launch_process.resend_request_to_pydevd(request)
+        else:
+            pause_response = base_schema.build_response(request)
+            self.write_to_client_message(pause_response)
 
-    def on_evaluate_request(self, request):
-        """
-        :param EvaluateRequest request:
-        """
+    def on_evaluate_request(self, request: EvaluateRequest):
         if self._launch_process is not None:
             if request.arguments.context == "repl":
                 pass
                 # i.e.: if not stopped anywhere we could send to the stdin...
                 # self._launch_process.send_to_stdin(request.arguments.expression)
-        self._launch_process.resend_request_to_robot(request)
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_setExceptionBreakpoints_request(self, request):
-        response = base_schema.build_response(request)
-        self.write_to_client_message(response)
+    def on_setExceptionBreakpoints_request(
+        self, request: SetExceptionBreakpointsRequest
+    ):
+        if self._run_in_debug_mode and self._launch_process is not None:
+            self._launch_process.resend_request_to_pydevd(request)
+        else:
+            response = base_schema.build_response(request)
+            self.write_to_client_message(response)
 
-    def on_setBreakpoints_request(self, request):
-        """
-        :param SetBreakpointsRequest request:
-        """
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import SourceBreakpoint
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import Breakpoint
+    def on_setBreakpoints_request(self, request: SetBreakpointsRequest):
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
-            SetBreakpointsResponseBody,
+            SetBreakpointsArguments,
         )
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import Source
 
-        if self._launch_process is None:
+        if self._launch_process is None or not self._run_in_debug_mode:
             # Just acknowledge that no breakpoints are valid.
             breakpoints = []
             if request.arguments.breakpoints:
@@ -224,35 +285,66 @@ class DebugAdapterComm(object):
             )
             return
 
-        self._launch_process.resend_request_to_robot(request)
+        arguments: SetBreakpointsArguments = request.arguments
+        source: Source = arguments.source
+        name = source.name
+        if name and name.lower().endswith((".py", ".pyw")):
+            self._launch_process.resend_request_to_pydevd(request)
+        else:
+            self._launch_process.resend_request_to_robot(request)
 
-    def on_continue_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_continue_request(self, request: ContinueRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_stepIn_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_stepIn_request(self, request: StepInRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_stepOut_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_stepOut_request(self, request: StepOutRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_stackTrace_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_stackTrace_request(self, request: StackTraceRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_next_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_next_request(self, request: NextRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_scopes_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_scopes_request(self, request: ScopesRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_variables_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_variables_request(self, request: VariablesRequest):
+        if self._launch_process:
+            self._launch_process.resend_request_to_stopped_target(
+                request, self.weak_stopped_target_comm()
+            )
 
-    def on_threads_request(self, request):
-        self._launch_process.resend_request_to_robot(request)
+    def on_threads_request(self, request: ThreadsRequest):
+        if self._launch_process:
+            if self._run_in_debug_mode:
+                self._launch_process.resend_request_to_pydevd(request)
+            else:
+                self._launch_process.resend_request_to_robot(request)
 
-    def write_to_client_message(self, protocol_message):
+    def write_to_client_message(self, protocol_message: BaseSchema):
         """
-        :param BaseSchema protocol_message:
+        :param protocol_message:
             Some instance of one of the messages in the debug_adapter.schema.
         """
         self.write_to_client_queue.put(protocol_message)
