@@ -7,6 +7,9 @@ OPTION_ROBOT_LANGUAGE_SERVER_TCP_PORT = "robot.language-server.tcp-port"
 OPTION_ROBOT_LANGUAGE_SERVER_ARGS = "robot.language-server.args"
 
 OPTION_ROBOT_VARIABLES = "robot.variables"
+
+OPTION_ROBOT_LINT_ROBOCOP_ENABLED = "robot.lint.robocop.enabled"
+
 OPTION_ROBOT_PYTHONPATH = "robot.pythonpath"
 OPTION_ROBOT_WORKSPACE_SYMBOLS_ONLY_FOR_OPEN_DOCS = (
     "robot.workspaceSymbolsOnlyForOpenDocs"
@@ -28,6 +31,7 @@ ALL_ROBOT_OPTIONS = frozenset(
         OPTION_ROBOT_LANGUAGE_SERVER_TCP_PORT,
         OPTION_ROBOT_LANGUAGE_SERVER_ARGS,
         OPTION_ROBOT_VARIABLES,
+        OPTION_ROBOT_LINT_ROBOCOP_ENABLED,
         OPTION_ROBOT_PYTHONPATH,
         OPTION_ROBOT_COMPLETION_SECTION_HEADERS_FORM,
         OPTION_ROBOT_WORKSPACE_SYMBOLS_ONLY_FOR_OPEN_DOCS,
